#!/usr/bin/perl
#
# addplaylist.pl - add playlist files to your database
#
# Copyright (c) by Andreas S. Krebs (andreas.krebs@web.de)
#
# this script adds playlists in m3u format to the muwed database
# 
# NOTE:
# all songs of the playlist have to exist in the database under
# the name used in the playlist or the insert will fail
#

use DBD::Pg;
use Getopt::Std;

$DBNAME="muwed";
$DBUSER="postgres";

chomp($BASEDIR=`pwd`);
$BASEDIR.="/";

$USAGE="Usage: addplaylist.pl [-u] playlist ...\n  -u: update values for existing playlists\n";

getopts('u') or die $USAGE;

if($#ARGV < 0)
{
  print "$USAGE";
  exit 0;
}

# try to connect to the database
$conn=DBI->connect("dbi:Pg:dbname=$DBNAME", "$DBUSER", "", { RaiseError => 1, AutoCommit => 0 })
        or die $DBI::errstr;

foreach $file (@ARGV)
{
  processFile($file);
}

$conn->commit() or die $DBI::errstr;
$conn->disconnect();

1;

sub processFile
{
  local($file)=shift(@_);
  local($title, $efile, $song, $query);
  $file=~s/^.\///;

  chomp($title=`basename $file`);
  $title=~ s/\..*$//g;

  print "processing: $file\n";    

  unless(open(FH, $file))
  {
     print STDERR "Can't open $file: $!\n";
     return;
  }
 
  #first check if the playlist already exists
  $query=$conn->prepare("SELECT * FROM playlists WHERE file=?");
  $query->execute($file);

  local($sqli,$rowref);
  if($query->rows == 0)     # was file not present in database ?
  {
    #let's create the playlist entry
    $query=$conn->prepare("INSERT INTO playlists VALUES (?, ?, NULL)");
    $query->execute($file, $title);
    
    #now get the playlist ID
    $query=$conn->prepare("SELECT * FROM playlists WHERE file=?");
    $query->execute($file);
    $rowref=$query->fetchrow_hashref();

    $plid=$rowref->{"id"};
  }
  elsif($opt_u eq 1)            # was an update for the file requested ?
  {
    $rowref=$query->fetchrow_hashref();
    $plid=$rowref->{"id"};

    #update the playlist entry
    $query=$conn->prepare("UPDATE playlists SET title=? WHERE id=?");
    $query->execute($title, $plid);

    #now delete ALL old playlist entries
    $query=$conn->prepare("DELETE FROM pl_entries WHERE pl_id=?");
    $query->execute($plid);
  }
  else
  {
    print "playlist already exists in database: use -u to update the playlist\n";
    return;
  }

  #now insert the songs into the entries
  $query=$conn->prepare("INSERT INTO pl_entries SELECT ?, id FROM songs s WHERE s.file=?");
  while(<FH>)
  {
    chomp;
  
    s/^$BASEDIR//g;
    $song=&removeSpaces($_);
    
    if($song ne "")
    {
      $query->execute($plid, $song);
    }
  }

  close(FH);
}

sub removeSpaces
{
  local($txt)=shift(@_);

  while(substr($txt, 0, 1) eq " " || substr($txt, 0, 1) eq "\t")
  {
    $txt=substr($txt, 1);
  }

  while(substr($txt, -1, 1) eq " " || substr($txt, -1, 1) eq "\t")
  {
    $txt=substr($txt, 0, -1);
  }

  $txt=~s/"/\\"/g;

  return $txt;
}

